#!/usr/bin/perl

# phase.pl 
# 
# Copyright (C) Center for Bioinformatics, Peking University <cope@mail.cbi.pku.edu.cn>
# 
# The software provided herein is free for ACADEMIC INSTRUCTION AND
# RESEARCH USE ONLY. You are free to download, copy, compile, study,
# and refer to the source code for any personal use of yours. Usage by
# you of any work covered by this license should not, directly or
# indirectly, enable its usage by any other individual or
# organization.
#
# You are free to make any modifications to the source covered by this
# license. You are also free to compile the source after modifying it
# and using the compiled product obtained thereafter in compliance
# with this License.  You may NOT under any circumstance copy,
# redistribute and/or republish the source or a work based on it
# (which includes binary or object code compiled from it) in part or
# whole without the permission of the authors.
# 
# If you intend to incorporate the source code, in part or whole, into
# any free or proprietary program, you need to explicitly write to the
# original author(s) to ask for permission via e-mail at
# cope@mail.cbi.pku.edu.cn.
# 
# Commercial licenses are available to legal entities, including
# companies and organizations (both for-profit and non-profit),
# requiring the software for general commercial use. To obtain a
# commercial license please, contact us via e-mail at
# cope@mail.cbi.pku.edu.cn.
#
# DISCLAIMER
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# THE ORIGINAL AUTHOR OF THE PROGRAM IS NOT LIABLE TO YOU FOR DAMAGES,
# INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING
# BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR
# LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM
# TO OPERATE WITH ANY OTHER PROGRAMS)




use strict;
use Getopt::Long;
## usage:

## config for shapeit
my $PATH_for_extractPIRs="/lustre/user/chengsj/15_Haplotype_Assembly/07_shapeit_static/extractPIRs/extractPIRs/bin";
my $PATH_for_shapeit2="/lustre/user/chengsj/15_Haplotype_Assembly/07_shapeit_static";
my $REF_panel="/lustre/user/chengsj/15_Haplotype_Assembly/07_shapeit_static/1000GP_Phase3/1000GP_Phase3";

## config for hapcut
my $path_for_hapcut="/lustre/user/chengsj/15_Haplotype_Assembly/03_HapCut/HAPCUT-v0.7";

## option
my $sample;
my $variant_file;
my $bam_file;
my $gender;
my $reference;
sub execute
{
    my $cmd = shift;
    system($cmd) == 0 or die "[corect.pl] ERROR executing command: $cmd\n.";
}

GetOptions ("sample|s=s" => \$sample,
	    "variant|v=s" => \$variant_file,
	    "bam|b:s"  => \$bam_file,
	    "gender|g:s" => \$gender,
	    "reference|r:s" => \$reference,
	   );

my $usage="perl phase.pl -sample sample -variant VCF.file -bam bamfile -gender [male|female]] -reference hg19.fa\n";
unless(-d "$sample" )
{
    mkdir $sample;
}

unless($variant_file && $bam_file && $gender && $reference && $sample)
{
    die $usage;
}

my $phased_snp_indel_file; ## final vcf format file
## haplotype reconstruction
print "Run haplotype inference\n";
execute("./script/hapcut.sh $path_for_hapcut $reference input/$variant_file input/$bam_file");
##run shapeit2
execute("perl script/change_vcf_format.pl input/$variant_file $gender > input/snp_indel_merged_for_shapeit.vcf");
if($gender eq 'male')
{
    execute("./script/run_shapeit2.sh $PATH_for_extractPIRs $PATH_for_shapeit2 $REF_panel input/$bam_file input/snp_indel_merged_for_shapeit.vcf $gender");
    `rm -f *mm`;
    `rm -f *log`;
}
elsif($gender eq 'female')
{
    execute("./script/run_shapeit2_for_female.sh $PATH_for_extractPIRs $PATH_for_shapeit2 $REF_panel input/$bam_file input/snp_indel_merged_for_shapeit.vcf $gender");
    `rm -f *mm`;
    `rm -f *log`;
}
else
{
    die "gender is wrong!";
}
## use hapcut to fix the result
execute("cat ./tmp_PIRs/*haplotype.txt.haps > ./tmp_PIRs/all.shapeit.result");
execute("perl ./script/use_hapcut_to_add_phase.pl input/$variant_file input/$variant_file.output_haplotype_file $gender > ./tmp_PIRs/add.haps.result");
execute("cat ./tmp_PIRs/all.shapeit.result ./tmp_PIRs/add.haps.result > $sample/final.phased.result");
## change final phased result into VCF format
print "change final phased result into VCF format\n";
open(VCF,"$sample/final.phased.result") or die;
open(FORMAT,">$sample/final.phased.vcf") or die; ##vcf format #chr pos id ref alt qual filter info format gt
print FORMAT "##fileformat=VCFv4.1\n";
while(my $line=<VCF>)
{
    chomp $line;
    my @info=split /\s+/,$line;
    print FORMAT "$info[0]\t$info[2]\t$info[1]\t$info[3]\t$info[4]\t.\t.\t.\tGT\t$info[5]|$info[6]\n";
}
close VCF;
close FORMAT;
$phased_snp_indel_file="final.phased.vcf";
## add X and Y
if($gender eq 'male')
{
    open(X,"./tmp_VCF/X.snp.vcf") or die;
    open(Y,"./tmp_VCF/Y.snp.vcf") or die;
    open(TO,">>$sample/final.phased.vcf");
    while(my $line=<X>)
    {
	chomp $line;
	if($line=~/^#/)
	{
	    next;
	}
	else
	{
	    my @info=split /\t/,$line;
	    print TO "$info[0]\t$info[1]\t$info[2]\t$info[3]\t$info[4]\t$info[5]\t$info[6]\t$info[7]\t$info[8]\t0|1\n";
	}
    }
    close X;
    while(my $line=<Y>)
    {
	chomp $line;
	if($line=~/^#/)
	{
	    next;
	}
	else
	{
	    my @info=split /\t/,$line;
	    print TO "$info[0]\t$info[1]\t$info[2]\t$info[3]\t$info[4]\t$info[5]\t$info[6]\t$info[7]\t$info[8]\t0|1\n";
	}
    }
    close Y;
	close TO;
}

print "final.phased.vcf is your final phased result.";
